#version 430 core
precision mediump float;

#define DOF_KERNEL_SIZE 8
out vec4 FragColor;

in vec2 TexCoords;

uniform sampler2D image;
uniform bool horizontal;
float dof_kernel[DOF_KERNEL_SIZE];

uniform float weight[5] = float[] (0.2270270270, 0.1945945946, 0.1216216216, 0.0540540541, 0.0162162162);

const int DOF_LCR_SIZE = DOF_KERNEL_SIZE * 2 - 1; // left-center-right (lllcrrr)
const int DOF_MEAN = DOF_LCR_SIZE / 2;

void makeBlurKernel(float sigma) {
    float sum = 0; // For accumulating the kernel values
    for (int x = DOF_MEAN; x < DOF_LCR_SIZE; x++)
    {
        dof_kernel[x - DOF_MEAN] = exp(-0.5 * pow((x - DOF_MEAN) / sigma, 2.0));
        // Accumulate the kernel values
        sum += dof_kernel[x - DOF_MEAN];
    }

    sum += sum - dof_kernel[0];

    // Normalize the kernel
    for (int x = 0; x < DOF_KERNEL_SIZE; x++) dof_kernel[x] /= sum;
}

void main()
{
     vec2 tex_offset = 1.0 / textureSize(image, 0); // gets size of single texel
     vec4 tex = texture(image, TexCoords);
     makeBlurKernel(12.0);
     vec3 result = tex.rgb * dof_kernel[0];

     if(horizontal)
     {
       for(int i = 1; i < DOF_KERNEL_SIZE; i++)
        {
            result += texture(image, TexCoords + vec2(tex_offset.x * i, 0.0)).rgb *  dof_kernel[i];
            result += texture(image, TexCoords - vec2(tex_offset.x * i, 0.0)).rgb *  dof_kernel[i];
         }
     }
     else
     {
       for(int i = 1; i < DOF_KERNEL_SIZE; i++)
        {
             result += texture(image, TexCoords + vec2(0.0, tex_offset.y * i)).rgb *  dof_kernel[i];
             result += texture(image, TexCoords - vec2(0.0, tex_offset.y * i)).rgb *  dof_kernel[i];
         }
     }
     FragColor = vec4(result, tex.a);
}
