#version 430 core
out vec4 FragColor;

in vec2 TexCoords;

uniform sampler2D bloomBlur;
uniform sampler2D scene;

uniform bool bloom = false;
uniform float exposure;
uniform float gamma;
uniform bool useACES = true;

//aces fitted curve taken from https://knarkowicz.wordpress.com/2016/01/06/aces-filmic-tone-mapping-curve/
vec3 ACESFilm(vec3 x)
{
    float a = 2.51f;
    float b = 0.03f;
    float c = 2.43f;
    float d = 0.59f;
    float e = 0.14f;
    return clamp((x*(a*x+b))/(x*(c*x+d)+e), 0, 1);
}

void main()
{
    //const float gamma = 2.2;
    vec4 color = texture(scene, TexCoords);
    vec3 hdrColor = color.rgb;
    vec3 bloomColor = texture(bloomBlur, TexCoords).rgb;
    if(bloom)
        hdrColor += bloomColor; // additive blending
		
    // tone mapping
    vec3 result = vec3(1.0) - exp(-hdrColor * exposure);
	
    if(useACES)
      result = ACESFilm(hdrColor);//;
	  
    // also gamma correct while we're at it
    result = pow(result, vec3(1.0 / gamma));
    FragColor = vec4(result, color.a);
}
