#version 430

precision mediump float;

#define DOF_KERNEL_SIZE 8
/* ^^^ definitions ^^^ */

out vec4 FragColor;

in vec2 texCoords;

uniform sampler2D image2; // dof
uniform sampler2D image3; // dof
uniform bool isHorizontal;
uniform bool ignore;

uniform float max_sigma = 15.0;
uniform float min_sigma = 0.0001;
uniform float bokehRadius;

float dof_kernel[DOF_KERNEL_SIZE];

vec4 tmp;
vec3 bloom_result;
vec3 dof_result;
float fdof;
int dofSize;
const int DOF_LCR_SIZE = DOF_KERNEL_SIZE * 2 - 1; // left-center-right (lllcrrr)
const int DOF_MEAN = DOF_LCR_SIZE / 2;

void makeDofKernel(float sigma) {

    float sum = 0; // For accumulating the kernel values
    for (int x = DOF_MEAN; x < DOF_LCR_SIZE; x++)
    {
        dof_kernel[x - DOF_MEAN] = exp(-0.5 * pow((x - DOF_MEAN) / sigma, 2.0));
        // Accumulate the kernel values
        sum += dof_kernel[x - DOF_MEAN];
    }

    sum += sum - dof_kernel[0];

    // Normalize the kernel
    for (int x = 0; x < DOF_KERNEL_SIZE; x++) dof_kernel[x] /= sum;

}

float Weigh(float coc, float radius)
{
  return clamp((coc-radius+2)/2, 0.0, 1.0);
}

void main() {

    //  vec3 c;
      vec2 texOffset = 1.0 / textureSize(image2, 0); // gets size of single texel
    //  vec2 o;
    //  float weight = 0;
      tmp = texture(image3, texCoords);

     fdof = tmp.a;
     //dofSize = clamp(int(tmp.a * DOF_KERNEL_SIZE), 1, DOF_KERNEL_SIZE);
     //if (dofSize % 2 == 0) dofSize++;
     makeDofKernel(12.0 * fdof + 1);
     dof_result = tmp.rgb * dof_kernel[0];

     if(isHorizontal)
     {
          for(int i = 1; i < DOF_KERNEL_SIZE; i++)
           {
              float o = dof_kernel[i];
              dof_result += texture(image2, texCoords + vec2(texOffset.x * i, 0.0)).rgb * o;
              dof_result += texture(image2, texCoords - vec2(texOffset.x * i, 0.0)).rgb * o;
           }
      } else
      {
          for(int i = 1; i < DOF_KERNEL_SIZE; i++)
          {
              float o = dof_kernel[i];
              dof_result += texture(image2, texCoords + vec2(0.0, texOffset.y * i)).rgb * o;
              dof_result += texture(image2, texCoords - vec2(0.0, texOffset.y * i)).rgb * o;
          }
     }
      FragColor = vec4(dof_result, fdof);
}
