#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent;

out VertexData {
	vec3 FragPos;
	vec2 uv;
	smooth vec3 normal_world;
	vec3 viewPos;
	vec3 FragPos_;
	vec3 viewPos_;
	vec4 FragPosLightSpace;
	float vis;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;
uniform vec3 camera_world;
uniform mat4 lightSpaceMatrix;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
	bool enabled;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
	bool enabled;
} pointL;

out struct DirectionLightTangent
{
	vec3 color;
	vec3 direction;
	vec3 directionN;
	bool enabled;
} dirLT;

out struct PointLightTangent {
	vec3 color;
	vec3 position;
	vec3 attenuation;
	bool enabled;
} pointLT;

const float density = 0.007;
const float gradient = 1.5f;

uniform float time = 1;

void main() {

	vec4 FragPos_ = modelMatrix * vec4(position, 1);
	 mat3 normMat = transpose(inverse(mat3(modelMatrix)));
	vec4 relPos = vec4(camera_world, 1) * FragPos_;

	vert.FragPos = FragPos_.xyz;
	vert.FragPos_ = vert.FragPos;
	vert.viewPos_ = camera_world;
	vert.normal_world = normMat * normal;
	vert.uv = uv;

	//calculate TBN matrix

	vec3 T = normalize(normMat * tangent);
  vec3 N = normalize(normMat * normal);
  T = normalize(T - dot(T, N) * N);
  vec3 B = cross(N, T);

  mat3 TBN = transpose(mat3(T, B, N));

	//setup Point Light in tangent space
	pointLT.color = pointL.color;
	pointLT.position = TBN * pointL.position;
	pointLT.attenuation = pointL.attenuation;
	pointLT.enabled = pointL.enabled;

	//setup directional light in tangent space
	dirLT.color = dirL.color;
	dirLT.direction =  dirL.direction;
	dirLT.enabled = dirL.enabled;
	dirLT.directionN = TBN* dirL.direction;

	vert.FragPos = TBN * vert.FragPos;
	vert.FragPosLightSpace = lightSpaceMatrix * FragPos_;
	vert.viewPos = TBN * camera_world;

	//fog calculation
  //float dist = length(relPos.xyz);
//	float vis = exp(-pow((dist * density), gradient));
//	vis = clamp(vis, 0.0, 1.0);
//  vert.vis = vis;
	gl_Position = viewProjMatrix * FragPos_;
}
