#version 430 core

out vec4 FragColor;
in vec2 TexCoords;
in vec3 normal_world;
in vec3 FragPos;
in vec3 ViewPos;

uniform vec3 materialAmbient;
uniform vec3 materialDiffuse;
uniform vec3 materialSpecular;
uniform float specularAlpha;
uniform bool enaDiff1;


uniform sampler2D tex;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 lightColor, vec3 materialDiff, vec3 lightColorSpec, vec3 materialSpec, float alpha, bool attenuate, vec3 attenuation, vec3 specColor, vec3 texColor, vec3 materialAmb) {
	float d = length(l);
	l = normalize(l);

	float att = 1.0;
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	float dnl = dot(n, l);
	float diff = max(0.0, dnl);
	float spec = pow(max(0.0, dot(r, v)), alpha);

  vec3 ambient = materialDiff * texColor * lightColor;
	vec3 diffuse = materialDiff * texColor * lightColor * diff;
	vec3 specular = materialSpec * specColor * lightColorSpec * spec;

	ambient *= att;
	diffuse *= att;
	specular *= att;
	return (ambient * (diffuse + specular)) ;
}

void main()
{
    vec4 myTex = texture(tex, TexCoords);
    vec4 color = vec4(materialAmbient, 1);
    vec3 texColor = materialAmbient;
    if(enaDiff1)
    {
      color = myTex;
      texColor = myTex.rgb;
    }

    vec3 v = vec3(0, 1,-0.5);
    vec3 specColor = materialSpecular;

    vec3 dir = vec3(-1, -1, -0.2);
    vec3 lColor = materialAmbient * 0.1;
    vec3 lSpecColor = materialAmbient * 0.1;
    color.rgb += phong(normal_world, dir, v, lColor , materialDiffuse , lSpecColor, materialSpecular , specularAlpha, false, vec3(1), specColor, texColor, materialAmbient);

    FragColor = color;
}
