#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 FragPos;
	vec3 normal_world;
	vec2 uv;
	mat3 TBN;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D texture_diffuse1;
uniform sampler2D texture_specular1;
uniform sampler2D texture_normal1;

const float Epsilon = 1e-10;


uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation, vec3 specColor, vec3 texColor) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	//vec3 halfway = normalize(l)
	return (diffuseF * diffuseC * texColor * max(0, dot(n, l)) + (specularF * specularC * specColor) * pow(max(0, dot(r, v)), alpha)) * att; 
}

vec3 RGBtoHSV(in vec3 RGB)
{
    vec4  P   = (RGB.g < RGB.b) ? vec4(RGB.bg, -1.0, 2.0/3.0) : vec4(RGB.gb, 0.0, -1.0/3.0);
    vec4  Q   = (RGB.r < P.x) ? vec4(P.xyw, RGB.r) : vec4(RGB.r, P.yzx);
    float C   = Q.x - min(Q.w, Q.y);
    float H   = abs((Q.w - Q.y) / (6.0 * C + Epsilon) + Q.z);
    vec3  HCV = vec3(H, C, Q.x);
    float S   = HCV.y / (HCV.z + Epsilon);
    return vec3(HCV.x, S, HCV.z);
}

vec3 HSVtoRGB(in vec3 HSV)
{
    float H   = HSV.x;
    float R   = abs(H * 6.0 - 3.0) - 1.0;
    float G   = 2.0 - abs(H * 6.0 - 2.0);
    float B   = 2.0 - abs(H * 6.0 - 4.0);
    vec3  RGB = clamp( vec3(R,G,B), 0.0, 1.0 );
    return ((RGB - 1.0) * HSV.y + 1.0) * HSV.z;
}

void main() {	
	vec3 normal = vert.normal_world;
	if( textureSize( texture_diffuse1, 0).x > 0){
		normal = texture(texture_normal1, vert.uv).rgb;
		// transform normal vector to range [-1,1]
		normal = normalize(normal * 2.0 - 1.0);  // this normal is in tangent space
		normal = normalize(vert.TBN * normal); 
	}

	vec3 n = normal;
	vec3 v = normalize(camera_world - vert.FragPos);

	vec3 texColor = vec3(1);
	vec3 specColor = vec3(1);

	if( textureSize( texture_diffuse1, 0).x > 0)
		texColor = texture(texture_diffuse1, vert.uv).rgb;
	
	if( textureSize( texture_specular1, 0).x > 0)
		specColor = texture(texture_specular1, vert.uv).rgb;

	//vec3 col_hsv = RGBtoHSV(texColor.rgb);
	//col_hsv.y *= 1.4;
	//texColor = vec3(HSVtoRGB(col_hsv));

	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color , materialCoefficients.y , dirL.color, materialCoefficients.z , specularAlpha, false, vec3(0), specColor, texColor);
			
	// add point light contribution
	//color.rgb += phong(n, pointL.position - vert.FragPos, v, pointL.color, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation, specColor, texColor);
	//color = vec4(specColor, 1);

	//color = vec4(normal, 1);
}