#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent; 

out VertexData {
	vec3 FragPos;
	vec3 normal_world;
	vec2 uv;
	mat3 TBN;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

void main() {
	vert.normal_world = normalMatrix * normal;
	vert.uv = uv;
	vec4 FragPos_ = modelMatrix * vec4(position, 1);
	vert.FragPos = FragPos_.xyz;

	mat3 normalMatrix = transpose(inverse(mat3(modelMatrix)));

	vec3 T = normalize(vec3(normalMatrix * tangent));
	vec3 B = normalize(vec3(normalMatrix * bitangent));
    vec3 N = normalize(vec3(normalMatrix * normal));
	

	T = normalize(T - dot(T, N) * N);
    B = cross(N, T);
	//if (dot(cross(N, T), B) < 0.0)
    //   T = T * -1.0;

	mat3 TBN = transpose(mat3(T, B, N) );
	vert.TBN = TBN;

	vert.FragPos  = vert.FragPos;
	gl_Position = viewProjMatrix * FragPos_;
}